package com.rxida.xilinedu.sys.common;

import com.rxida.xilinedu.sys.dao.UserDao;
import com.rxida.xilinedu.sys.entity.Permission;
import com.rxida.xilinedu.sys.entity.Role;
import com.rxida.xilinedu.sys.entity.User;
import com.rxida.xilinedu.sys.utils.Encodes;
import org.apache.shiro.authc.*;
import org.apache.shiro.authc.credential.HashedCredentialsMatcher;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.crypto.hash.Sha256Hash;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.util.ByteSource;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Component("authorizer")
public class SystemAuthorizingRealm extends AuthorizingRealm {
    @Resource
    private UserDao userDao;

    public SystemAuthorizingRealm() {
        this.setCachingEnabled(false);
    }

    //角色权限和对应权限添加
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
        //获取登录用户名
        String name = (String) principalCollection.getPrimaryPrincipal();
        //查询用户名称
        User user = userDao.findByUsername(name);
        //添加角色和权限
        SimpleAuthorizationInfo simpleAuthorizationInfo = new SimpleAuthorizationInfo();
        for (Role role : user.getRoles()) {
            //添加角色
            simpleAuthorizationInfo.addRole(role.getName());
            for (Permission permission : role.getPermissions()) {
                //添加权限
                simpleAuthorizationInfo.addStringPermission(permission.getPermission());
            }
        }
        return simpleAuthorizationInfo;
    }

    //用户认证
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
        //加这一步的目的是在Post请求的时候会先进认证，然后在到请求
        if (authenticationToken.getPrincipal() == null) {
            return null;
        }
        //获取用户信息
        User u = (User) authenticationToken.getPrincipal();
        User user = userDao.findByUsername(u.getUsername());
        if (user == null) {
            //这里返回后会报出对应异常
            return null;
        } else {
            byte[] salt = Encodes.decodeHex(user.getPassword().substring(0, 16));
            return new SimpleAuthenticationInfo(
                    user,
                    user.getPassword().substring(16),
                    ByteSource.Util.bytes(salt),
                    getName());
        }
    }
}
