package com.rxida.xilinedu.sys.service;

import com.rxida.xilinedu.sys.common.CrudDao;
import com.rxida.xilinedu.sys.entity.BaseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Transactional(readOnly = true)
@Service
public class BaseService<D extends CrudDao<E>, E extends BaseEntity> {
    protected D dao;

    public List<E> findList(E entity) {
        return dao.findList(entity);
    }

    public E get(E entity) {
        return dao.get(entity);
    }

    @Transactional
    public int delete(E entity) {
        return dao.delete(entity);
    }

    @Transactional
    public int save(E entity) {
        if (entity.getId() == null || "".equals(entity.getId())) {
            entity.preInsert();
            return dao.insert(entity);
        } else {
            entity.preUpdate();
            return dao.update(entity);
        }
    }

    @Transactional
    public void batchDelete(List<E> list) {
        dao.batchDelete(list);
    }

    @Transactional
    public void batchInsert(List<E> list) {
        list.forEach(item -> item.preInsert());
        dao.batchInsert(list);
    }

    @Transactional
    public void batchUpdate(List<E> list) {
        list.forEach(item -> item.preUpdate());
        dao.batchUpdate(list);
    }
}
